@@ -4,21 +4,100 @@ import gzip
 import os
 import sys
 import sysconfig
+from enum import IntEnum
 from types import TracebackType
-from typing import TYPE_CHECKING, Any, List, Optional, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, List, Optional, Tuple, Type
 
 if TYPE_CHECKING:
     from PIL import Image
 
 from ._rlottiecommon import LOTLayerNode, LOTMarkerList
 
-# References: rlottie/inc/rlottie.h
+# References: rlottie/inc/rlottie_capi.h
+
+
+def _load_lib_with_prefix_suffix(
+    lib_prefix: str, lib_suffix: str
+) -> Optional[ctypes.CDLL]:
+    package_dir = os.path.dirname(__file__)
+    rlottie_lib_name = lib_prefix + "rlottie" + lib_suffix
+    rlottie_lib_path_local = os.path.join(package_dir, rlottie_lib_name)
+
+    if os.path.isfile(rlottie_lib_path_local):
+        rlottie_lib_path = rlottie_lib_path_local
+    elif os.path.isfile(rlottie_lib_name):
+        rlottie_lib_path = os.path.abspath(rlottie_lib_name)
+    else:
+        rlottie_lib_path = rlottie_lib_name
+
+    try:
+        return ctypes.cdll.LoadLibrary(rlottie_lib_path)
+    except OSError:
+        return None
+
+
+def _load_lib(rlottie_lib_path: Optional[str] = None) -> Optional[ctypes.CDLL]:
+    if rlottie_lib_path:
+        try:
+            return ctypes.cdll.LoadLibrary(rlottie_lib_path)
+        except OSError:
+            return None
+
+    if sys.platform.startswith(("win32", "cygwin", "msys", "os2")):
+        lib = _load_lib_with_prefix_suffix("", ".dll")
+    elif sys.platform.startswith("darwin"):
+        lib = _load_lib_with_prefix_suffix("lib", ".dylib")
+    else:
+        lib = _load_lib_with_prefix_suffix("lib", ".so")
+
+    if lib:
+        return lib
+
+    lib_suffixes: List[str] = []
+    shlib_suffix = sysconfig.get_config_var("SHLIB_SUFFIX")
+    if isinstance(shlib_suffix, str):
+        lib_suffixes.append(shlib_suffix)
+    if sys.platform.startswith(("win32", "cygwin", "msys", "os2")):
+        lib_prefixes = ("", "lib")
+    elif sys.platform.startswith("darwin"):
+        lib_prefixes = ("lib", "")
+    else:
+        lib_prefixes = ("lib", "")
+    lib_suffixes.extend([".so", ".dll", ".dylib"])
+
+    for lib_prefix in lib_prefixes:
+        for lib_suffix in set(lib_suffixes):
+            lib = _load_lib_with_prefix_suffix(lib_prefix, lib_suffix)
+            if lib:
+                return lib
+
+    return None
+
+
+RLOTTIE_LIB = _load_lib()
 
 
 class _LottieAnimationPointer(ctypes.c_void_p):
     pass
 
 
+class LottieAnimationProperty(IntEnum):
+    LOTTIE_ANIMATION_PROPERTY_FILLCOLOR = 0
+    LOTTIE_ANIMATION_PROPERTY_FILLOPACITY = 1
+    LOTTIE_ANIMATION_PROPERTY_STROKECOLOR = 2
+    LOTTIE_ANIMATION_PROPERTY_STROKEOPACITY = 3
+    LOTTIE_ANIMATION_PROPERTY_STROKEWIDTH = 4
+    LOTTIE_ANIMATION_PROPERTY_TR_ANCHOR = 5
+    LOTTIE_ANIMATION_PROPERTY_TR_POSITION = 6
+    LOTTIE_ANIMATION_PROPERTY_TR_SCALE = 7
+    LOTTIE_ANIMATION_PROPERTY_TR_ROTATION = 8
+    LOTTIE_ANIMATION_PROPERTY_TR_OPACITY = 9
+
+    @classmethod
+    def from_param(cls, obj: int) -> int:
+        return int(obj)
+
+
 class LottieAnimation:
     def __init__(
         self,
@@ -46,52 +125,18 @@ class LottieAnimation:
             )
 
     def _load_lib(self, rlottie_lib_path: Optional[str] = None) -> None:
-        if rlottie_lib_path:
-            self.rlottie_lib_path = rlottie_lib_path
+        if rlottie_lib_path is None:
+            if RLOTTIE_LIB is None:
+                raise OSError("Could not load rlottie library")
+            else:
+                self.rlottie_lib = RLOTTIE_LIB
+                return
 
-            try:
-                self.rlottie_lib = ctypes.CDLL(self.rlottie_lib_path)
-            except OSError:
-                raise OSError(
-                    "Unable to load rlottie library at: " + self.rlottie_lib_path
-                )
-
-            return None
-
-        lib_suffixes: List[str] = []
-        shlib_suffix = sysconfig.get_config_var("SHLIB_SUFFIX")
-        if isinstance(shlib_suffix, str):
-            lib_suffixes.append(shlib_suffix)
-        if sys.platform.startswith(("win32", "cygwin", "msys", "os2")):
-            lib_prefixes = ("", "lib")
-        elif sys.platform.startswith("darwin"):
-            lib_prefixes = ("lib", "")
+        rlottie_lib = _load_lib(rlottie_lib_path)
+        if rlottie_lib is None:
+            raise OSError(f"Could not load rlottie library from {rlottie_lib_path}")
         else:
-            lib_prefixes = ("lib", "")
-        lib_suffixes.extend([".so", ".dll", ".dylib"])
-
-        package_dir = os.path.dirname(__file__)
-
-        attempted_lib_paths: List[str] = []
-        for lib_prefix in lib_prefixes:
-            for lib_suffix in lib_suffixes:
-                rlottie_lib_name = lib_prefix + "rlottie" + lib_suffix
-                rlottie_lib_path_local = os.path.join(package_dir, rlottie_lib_name)
-
-                if os.path.isfile(rlottie_lib_path_local):
-                    self.rlottie_lib_path = rlottie_lib_path_local
-                elif os.path.isfile(rlottie_lib_name):
-                    self.rlottie_lib_path = os.path.abspath(rlottie_lib_name)
-                else:
-                    self.rlottie_lib_path = rlottie_lib_name
-
-                try:
-                    self.rlottie_lib = ctypes.CDLL(self.rlottie_lib_path)
-                    return None
-                except OSError:
-                    attempted_lib_paths.append(self.rlottie_lib_path)
-
-        raise OSError("Unable to load rlottie library at: " + str(attempted_lib_paths))
+            self.rlottie_lib = rlottie_lib
 
     def __del__(self) -> None:
         if self.rlottie_lib:
@@ -566,7 +611,7 @@ class LottieAnimation:
         return bytes(self.async_buffer_c)
 
     def lottie_animation_property_override(
-        self, _type: str, keypath: str, *args: Union[ctypes.c_float, ctypes.c_int]
+        self, _type: LottieAnimationProperty, keypath: str, *args: ctypes.c_double
     ) -> None:
         """
         Request to change the properties of this animation object.
@@ -574,60 +619,98 @@ class LottieAnimation:
         Keypath should conatin object names separated by (.)
         and can handle globe(**) or wildchar(*)
 
+        .. list-table:: Possible values of _types and args
+            :header-rows: 1
+
+            * - _type LottieAnimationProperty
+              - No. of args
+              - args value
+              - Description
+            * - LOTTIE_ANIMATION_PROPERTY_FILLCOLOR
+              - 3 args
+              - [0 ... 1]
+              - Color property of Fill object:
+            * - LOTTIE_ANIMATION_PROPERTY_FILLOPACITY
+              - 1 args
+              - [0 ... 100]
+              - Opacity property of Fill object
+            * - LOTTIE_ANIMATION_PROPERTY_STROKECOLOR
+              - 3 args
+              - [0 ... 1]
+              - Color property of Stroke object
+            * - LOTTIE_ANIMATION_PROPERTY_STROKEOPACITY
+              - 1 args
+              - [0 ... 100]
+              - Opacity property of Stroke object
+            * - LOTTIE_ANIMATION_PROPERTY_STROKEWIDTH
+              - 1 args
+              - [0 ... +inf]
+              - Stroke width property of Stroke object
+            * - LOTTIE_ANIMATION_PROPERTY_TR_ANCHOR
+              - 0 args
+              - Any
+              - Transform Anchor property of Layer and Group object
+            * - LOTTIE_ANIMATION_PROPERTY_TR_POSITION
+              - 2 args
+              - Any
+              - Transform Position property of Layer and Group object
+            * - LOTTIE_ANIMATION_PROPERTY_TR_SCALE
+              - 2 args
+              - Any
+              - Transform Scale property of Layer and Group object
+            * - LOTTIE_ANIMATION_PROPERTY_TR_ROTATION
+              - 1 args
+              - [0 ... 360]
+              - Transform Scale property of Layer and Group object
+            * - LOTTIE_ANIMATION_PROPERTY_TR_OPACITY
+              - 0 args
+              - Any
+              - Transform Opacity property of Layer and Group object
+
         Example:
-        lottie_animation_property_override(
-        "LOTTIE_ANIMATION_PROPERTY_FILLCOLOR",
-        "layer1.group1.fill1",
-        ctypes.c_float(1.0),
-        ctypes.c_float(0.0),
-        ctypes.c_float(0.0)
-        )
+        To change fillcolor property of fill1 object in the
+        layer1->group1->fill1 hirarchy to RED color:
 
-        :param str _type: Property type.
+        .. code-block:: python
+
+            lottie_animation_property_override(
+                LottieAnimationProperty.LOTTIE_ANIMATION_PROPERTY_FILLCOLOR,
+                "layer1.group1.fill1",
+                ctypes.c_double(1.0),
+                ctypes.c_double(0.0),
+                ctypes.c_double(0.0)
+            )
+
+        If all the color property inside group1 needs to be changed to GREEN color:
+
+        .. code-block:: python
+
+            lottie_animation_property_override(
+                LottieAnimationProperty.LOTTIE_ANIMATION_PROPERTY_FILLCOLOR,
+                "**.group1.**",
+                ctypes.c_double(0.0),
+                ctypes.c_double(1.0),
+                ctypes.c_double(0.0)
+            )
+
+        :param LottieAnimationProperty _type: Property type.
         :param str keypath: Specific content of target.
-        :param Union[ctypes.c_float, ctypes.c_int] *args: Property values.
+        :param ctypes.c_double *args: Property values.
         """
-        _type_enum = [
-            # Color property of Fill object
-            # value type is float [0 ... 1]
-            "LOTTIE_ANIMATION_PROPERTY_FILLCOLOR",
-            # Opacity property of Fill object
-            # value type is float [ 0 .. 100]
-            "LOTTIE_ANIMATION_PROPERTY_FILLOPACITY",
-            # Color property of Stroke object
-            # value type is float [0 ... 1]
-            "LOTTIE_ANIMATION_PROPERTY_STROKECOLOR",
-            # Opacity property of Stroke object
-            # value type is float [ 0 .. 100]
-            "LOTTIE_ANIMATION_PROPERTY_STROKEOPACITY",
-            # stroke with property of Stroke object
-            # value type is float
-            "LOTTIE_ANIMATION_PROPERTY_STROKEWIDTH",
-            # Transform Anchor property of Layer and Group object
-            # value type is int
-            "LOTTIE_ANIMATION_PROPERTY_TR_ANCHOR",
-            # Transform Position property of Layer and Group object
-            # value type is int
-            "LOTTIE_ANIMATION_PROPERTY_TR_POSITION",
-            # Transform Scale property of Layer and Group object
-            # value type is float range[0 ..100]
-            "LOTTIE_ANIMATION_PROPERTY_TR_SCALE",
-            # Transform Scale property of Layer and Group object
-            # value type is float. range[0 .. 360] in degrees
-            "LOTTIE_ANIMATION_PROPERTY_TR_ROTATION",
-            # Transform Opacity property of Layer and Group object
-            # value type is float [ 0 .. 100]
-            "LOTTIE_ANIMATION_PROPERTY_TR_OPACITY",
+        argtypes: List[Any] = [
+            _LottieAnimationPointer,
+            LottieAnimationProperty,
+            ctypes.c_wchar_p,
         ]
+        for _ in args:
+            argtypes.append(ctypes.c_double)
 
-        try:
-            _type_index = _type_enum.index(_type)
-        except IndexError:
-            raise IndexError("Invalid _type")
+        self.rlottie_lib.lottie_animation_property_override.argtypes = argtypes
+        self.rlottie_lib.lottie_animation_property_override.restype = ctypes.c_void_p
 
         self.rlottie_lib.lottie_animation_property_override(
             self.animation_p,
-            ctypes.c_int(_type_index),
+            _type,
             ctypes.c_wchar_p(keypath),
             *args,
         )
